'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ulid = require('ulid');

var _require = require('./pool'),
    ActionPool = _require.ActionPool;

var pools = {};

var LockedPool = function (_ActionPool) {
  (0, _inherits3.default)(LockedPool, _ActionPool);

  function LockedPool() {
    (0, _classCallCheck3.default)(this, LockedPool);

    var _this = (0, _possibleConstructorReturn3.default)(this, (LockedPool.__proto__ || Object.getPrototypeOf(LockedPool)).call(this));

    _this.id = ulid();
    _this.locks = {};
    return _this;
  }

  (0, _createClass3.default)(LockedPool, [{
    key: 'incrementLock',
    value: function incrementLock(model) {
      if (!(model.id in this.locks)) this.locks[model.id] = 0;

      this.locks[model.id] += 1;
    }
  }, {
    key: 'decrementLock',
    value: function decrementLock(model) {
      if (!(model.id in this.locks)) return;

      this.locks[model.id] -= 1;

      if (this.locks[model.id] === 0) {
        delete this.locks[model.id];
      }
    }
  }, {
    key: 'locked',
    value: function locked(model) {
      return model.id in this.locks;
    }
  }, {
    key: 'hasAnyLock',
    value: function hasAnyLock() {
      return Object.keys(this.locks) > 0;
    }
  }, {
    key: 'lock',
    value: function lock() {
      var _this2 = this;

      var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

      models.forEach(function (model) {
        return _this2.incrementLock(model);
      });
    }
  }, {
    key: 'unlock',
    value: function unlock() {
      var _this3 = this;

      var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

      models.forEach(function (model) {
        return _this3.decrementLock(model);
      });
    }
  }]);
  return LockedPool;
}(ActionPool);

var createPool = function createPool() {
  var pool = new LockedPool();

  pools[pool.id] = pool;
  return pool;
};

var findPool = function findPool(models) {
  return Object.keys(pools).find(function (poolId) {
    return models.some(function (model) {
      return pools[poolId].locked(model);
    });
  });
};

var getOrCreatePool = function getOrCreatePool(models) {
  return findPool(models) || createPool();
};

var cleanPool = function cleanPool(pool) {
  if (!pool.hasAnyLock()) delete pools[pool.id];
};

var dispatch = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(action, models) {
    var pool, res;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            pool = getOrCreatePool(models);


            pool.lock(models);
            _context.next = 4;
            return pool.delegate(action);

          case 4:
            res = _context.sent;

            pool.unlock(models);

            cleanPool(pool);

            return _context.abrupt('return', res);

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  }));

  return function dispatch(_x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

module.exports = {
  dispatch: dispatch
};
